/*
		Obt: para funciones que obtienen datos
		Asig: para funciones que asignan datos
		Impr: para funciones que imprimen datos en pantalla

*/


/*
	Guarda informacin de un archivo de audio para poder ser usada convenientemente
	'na' es llenado cuando se abre el archivo de audio usando 'mus_audio_open_imput'
	que devuelve un nmero nico de archivo o -1 si no se puede abrir el archivo.
	'muestras' y 'mu_can' representan mnbos un valor en muestras; el primero se
	refiere al nmero total de muestras que tiene el archivo mientras que el segundo
	a la cantidad de muestras por canal.
*/
typedef struct 
{
  int na;                /* numero de archivo asociado, o -1 si se falla la apertura del archivo */
  int canales;				   /* canales del audio */
  int frec_muestreo;     /* frecuencia de muestreo */
  off_t muestras;        /* muestras en total que tiene el archivo de audio */
  off_t mu_can;          /* se refiere a la cantidad de muestras en cada canal */
  float duracion;        /* la duracin en segundos del archivo de audio */
  int formato;           /* el formato que tiene el archivo de audio */
  int cabecera;
  char *comentario;      /* el comentario que pueda tener el archivo de audio, de otro modo es NULL */
} DAUDIO;

#define TAM_BUFFER 1000   /*  tamao de un espacio de memoria a usar */


/* 
	chequea que la entrada de la lnea de comandos tenga el nmero de argumentos necesariso 

	args : la cantidad de argumentos que el usuario ingres.
	nargs : la cantidad argumentos necesarios para correr sta aplicacin.
	msj : el mensaje que se va a mostrar, dependiendo de la aplicacin.
	nomb_exe : nombre del ejecutable de sta aplicacin.
*/
int ChequearEntrada(int args, int nargs, char *msj, char *nomb_exe)
{
  if(args < nargs) 
		{
    fprintf(stderr, "\n  %s: insuficientes argumentos [%d] \n  %s", nomb_exe, args, msj);
    fprintf(stdout, "\n\n Presione aceptar para terminar el programa ..."); 
    return -1;
		}
  return 0; 
}


/* 
	Imprime en la consola informacin del archivo de audio guardados en 'dau'.
	Tambin se le pasa el nombre del archivo para obtener el tipo de la cabecera 
	y el formato del audio, stos datos no estn guardados en la estructura DAUDIO. 
	Por ltmo, se permite pasar un mensaje a ser mostrado junto con los datos como ttulo.

	msj: es utilizado como un ttulo que se agrega ntes del nombre del archvio de audio.
*/
void ImprDatosDeAudio(char *narch, DAUDIO *dau, char *msj)
{
	fprintf(stdout, "\n%s: %s:\n\n  F.MUESTREO: \t%d  m/s\n  CANALES: \t%d\n  DURACION: \t%f  seg\n  MUESTRAS: \t%d\n  MUES/CAN: \t%d\n",
		msj, narch, dau->frec_muestreo, dau->canales, dau->duracion, dau->muestras, dau->mu_can);

  fprintf(stdout, "  TIPO: \t%s\n  FORMATO: \t%s\n  COMENTARIO: \t%s\n", 
								  mus_header_type_name(mus_sound_header_type(narch)),
									mus_data_format_name(mus_sound_data_format(narch)),
									dau->comentario);
}


/* 
	abre un archivo para salida, o sea que puede ser modificado, usando como
	datos la estructura DAUDIO, que contiene toda la info necesaria, luego imprime 
	sus daots.
*/
void AbrirAudioSalida(char* nombre, DAUDIO *datos, char* msj)
{
  datos->na = mus_sound_open_output(nombre, datos->frec_muestreo, datos->canales, 
				                            datos->formato, datos->cabecera, datos->comentario);
  if (datos->na != -1)
		ImprDatosDeAudio(nombre, datos, msj);
  else 
		{
    fprintf(stderr, "%s: %s\n", nombre, strerror(errno));
    fprintf(stdout,"\n\n Presione aceptar para terminar el programa ..."); 
    getchar();
		}
}


/* Crea memoria para ser usado por el snd. Toma en cuenta el nmero de canales y el
tamao del BUFFER, creo */
mus_sample_t **SndAsignarMemoria(DAUDIO *dau, long int tam)
{
  mus_sample_t **buf;
  int i;

	/* CALLOC es una macro creada en sndlib.h para reservar memoria	*/
  buf = (mus_sample_t **)CALLOC(dau->canales, sizeof(int *));
  for(i = 0; i < dau->canales; i++) 
		buf[i] = (mus_sample_t *)CALLOC(tam, sizeof(mus_sample_t));

  return buf;
}


/* Libera la memoria creada anteriormente para el snd */
void SndLiberarMemoria(DAUDIO *dau, mus_sample_t **buf)
{
  int i;
		/* Debido a que sta memoria fue creada con CALLOC se usa FREE para liberarla */
  for(i = 0; i < dau->canales; i++) 
		FREE(buf[i]);
  FREE(buf);
}


