

#include <math.h>

#define DOS_PI (3.14159265358979323846264338327 * 2)

typedef struct 
{
  double  indice;
  double  incr;
} OSCIL;


/*
	Crea una tabla sinusoidal. Se le pasa el pco de amplitud, en valores crudos;
	la fase inicial, entre cero y uno; y la longitud de la tabla en muestras
*/
float *CrearTablaSeno(float fase, int longitud)
{
  float *tp = NULL;                           /* todos las 'alturas' de las muestras */
  int i;
  float radian = 0.0;                           /* fase por defecto, ser modificada por 'phase'*/
  float incr = DOS_PI / ((float)(longitud) + 1);   /* la fase se incrementa por ste valor */
    
  tp = (float *)malloc(longitud * sizeof(float) + 1);   /* memoria del largo de 'length' */
  radian += fase * DOS_PI;                               /* la fase se indica de 0 a 1 */
  
  for(i = 0; i < longitud; i++) 
		{
    tp[i] = sin(radian);        /* simplemente hacemos una tabla sinusoidal */
    radian += incr;                   /* incrementamos la fase */
		}
  return(tp);                        /* devolvemos la tabla creada */
}

OSCIL *IniciarOscilador(int largo, float fase)
{
  OSCIL *oscil;

  oscil = (OSCIL *)malloc(sizeof(OSCIL));
		 /* el incremento tiene que ver con cuantas muestras se avanza
		 en cada nueva toma de datos. */
  oscil->incr = 0.0;

	/* el ndice comienza en un lugar de la tabla creada, que est 
	representado por la fase como una fraccin de la misma.	*/
  oscil->indice = (double)(fase * (double)largo);
  
  return(oscil);
}


float ValorActualOscilador(OSCIL *oscil, float frec, float amp, float *tabla, int largo, int frec_mues)
{
	float salida;
  int voscil = 0;   /* valor que se obtiene del oscilador */

		/* Convierte a incremento la frecuencia */
  oscil->incr = (double)frec * ((double)largo / (double)frec_mues);
  voscil = (int)(oscil->indice + 1.0);
  while (voscil >= largo) 
		voscil -= largo;
  
			/* interpolacion Lineal */
  salida = amp * (tabla[(int)oscil->indice] + 	
						((tabla[voscil] - tabla[(int)oscil->indice]) * (oscil->indice - (int)oscil->indice)));

  oscil->indice += oscil->incr;
  
		/*
		contorlan si el indice va ms all de un ciclo del oscilador
		o ntes del comienzo del oscilador. De sta manera siempre 
		se mantiene dentro de un ciclo del oscilador
		*/
  while (oscil->indice >= (float)largo) 
		oscil->indice -= (double)largo;
  while (oscil->indice < 0.0)
    oscil->indice += (double)largo;
  
  return salida;
}

/*
float ValorActualOscilador(int incr, float frec, float amp, float fase)
{
	float val;
	val = amp * sin(DOS_PI * frec * incr + fase);
	return val;
}
*/
